"use client";

import "@fontsource/roboto/300.css";
import "@fontsource/roboto/400.css";
import "@fontsource/roboto/500.css";
import "@fontsource/roboto/700.css";
import { Box, Button, Container, Stack, Typography } from "@mui/material";
import { ProjectCard } from "@/components/cards/ProjectCard";
import { DataServiceCard } from "@/components/cards/DataServiceCard";
import { CardList } from "@/components/cards/CardList";
import { useRouter } from "next/navigation";
import AddIcon from "@mui/icons-material/Add";
import { useState } from "react";
import { ConfirmModal } from "@/components/ConfirmModal";
import { useGetDataServiceList } from "@/hooks/useGetDataServiceList";
import { useGetProjectList } from "@/hooks/useGetProjectList";
import {
  useApiDeleteDataService,
  useApiDeleteProject,
  useApiUpdateProjectState,
} from "@/api/cubicweb";
import { useHandleAuthErrors } from "@/hooks/useHandleAuthErrors";

export default function Home() {
  const { data: projects, loading: projectsLoading } = useGetProjectList();
  const { data: dataServices, loading: dataServicesLoading } =
    useGetDataServiceList();
  const router = useRouter();
  const [modal, setModal] = useState<{
    visible: boolean;
    text?: string;
    eid?: number;
    type?: "deleteDataService" | "deleteProject" | "toggle";
  }>({ visible: false });
  const handleAuthErrors = useHandleAuthErrors();

  const deleteDataService = useApiDeleteDataService();
  const deleteProject = useApiDeleteProject();
  const updateProjectState = useApiUpdateProjectState();

  return (
    <Container>
      <Stack spacing={4}>
        <Stack spacing={2}>
          <Stack direction={"row"} alignItems={"center"}>
            <Box flex={1}>
              <Typography variant="h3">Projets</Typography>
            </Box>
            <Box>
              <Button
                startIcon={<AddIcon />}
                onClick={() => {
                  router.push("/project/new");
                }}
              >
                Ajouter
              </Button>
            </Box>
          </Stack>
          <CardList loading={projectsLoading} emptyText="Aucun projet">
            {projects?.map((p, i) => (
              <ProjectCard
                key={i}
                name={p.name}
                activated={p.activated}
                virtuoso_url={p.virtuoso_url}
                onOpen={() => {
                  router.push("/project/" + p.eid);
                }}
                onDelete={() =>
                  setModal({
                    visible: true,
                    text: `Voulez vous vraiment supprimer le projet '${p.name}' ?`,
                    eid: p.eid,
                    type: "deleteProject",
                  })
                }
                onToggle={() =>
                  setModal({
                    visible: true,
                    text: p.activated
                      ? `Voulez vous vraiment désactiver le projet '${p.name}' ?`
                      : `Voulez vous vraiment activer le projet '${p.name}' ?`,
                    eid: p.eid,
                    type: "toggle",
                  })
                }
              />
            ))}
          </CardList>
        </Stack>
        <Stack spacing={2}>
          <Stack direction={"row"} alignItems={"center"}>
            <Box flex={1}>
              <Typography variant="h3">Data Services</Typography>
            </Box>
            <Box>
              <Button
                startIcon={<AddIcon />}
                onClick={() => {
                  router.push("/data-service/new");
                }}
              >
                Ajouter
              </Button>
            </Box>
          </Stack>
          <CardList
            loading={dataServicesLoading}
            emptyText="Aucun data service"
          >
            {dataServices?.map((d, i) => (
              <DataServiceCard
                key={i}
                name={d.name}
                data_url={d.data_url}
                refresh_prediod={d.refresh_period}
                onOpen={() => {
                  router.push("/data-service/" + d.eid);
                }}
                onDelete={() =>
                  setModal({
                    visible: true,
                    text: `Voulez vous vraiment supprimer le data service '${d.name}' ?`,
                    eid: d.eid,
                    type: "deleteDataService",
                  })
                }
              />
            ))}
          </CardList>
        </Stack>
      </Stack>
      <ConfirmModal
        open={modal.visible}
        text={modal?.text ?? ""}
        onClose={() => setModal((prev) => ({ ...prev, visible: false }))}
        onAccept={async () => {
          try {
            if (modal.type === "toggle") {
              const project = projects?.find((p) => p.eid === modal.eid);
              if (project && modal.eid) {
                await updateProjectState(modal.eid, !project.activated);
                project.activated = !project.activated;
              }
            } else if (modal.type === "deleteProject" && modal.eid) {
              await deleteProject(modal.eid);
              projects?.splice(
                projects?.findIndex((p) => p.eid === modal.eid),
                1,
              );
            } else if (modal.type === "deleteDataService" && modal.eid) {
              await deleteDataService(modal.eid);
              dataServices?.splice(
                dataServices?.findIndex((p) => p.eid === modal.eid),
                1,
              );
            }
          } catch (e) {
            handleAuthErrors(e);
          }
        }}
      />
    </Container>
  );
}
