import {
  Stack,
  TextField,
  Box,
  FormControl,
  InputLabel,
  Select,
  MenuItem,
  FormHelperText,
} from "@mui/material";
import SaveIcon from "@mui/icons-material/Save";
import { DataService } from "@/types";
import { Controller, SubmitHandler, useForm } from "react-hook-form";
import { useState } from "react";
import { useRouter } from "next/navigation";
import { LoadingButton } from "@mui/lab";
import { useHandleAuthErrors } from "@/hooks/useHandleAuthErrors";
import {
  useApiCreateDataService,
  useApiUpdateDataService,
} from "@/api/cubicweb";

export interface DataServiceFormProps {
  dataService?: DataService;
}

export function DataServiceForm({ dataService }: DataServiceFormProps) {
  const [loading, setLoading] = useState(false);
  const {
    handleSubmit,
    control,
    formState: { isDirty },
    reset,
  } = useForm<DataService>({
    defaultValues: dataService
      ? dataService
      : {
          refresh_period: "weekly",
        },
  });
  const handleAuthErrors = useHandleAuthErrors();
  const createDataService = useApiCreateDataService();
  const updateDataService = useApiUpdateDataService();

  const router = useRouter();

  const onSubmit: SubmitHandler<DataService> = async (data) => {
    setLoading(true);
    try {
      if (data.eid === undefined) {
        const result = await createDataService(data);
        const eid = result[0][0];
        setLoading(false);
        router.push(`/data-service/${eid}`);
      } else {
        await updateDataService(data);
        reset(data);
        setLoading(false);
      }
    } catch (e) {
      handleAuthErrors(e);
    }
  };

  return (
    <form onSubmit={handleSubmit(onSubmit)}>
      <Stack spacing={2}>
        {!dataService ? (
          <Controller
            name="name"
            control={control}
            rules={{ required: true }}
            render={({ field, fieldState: { error } }) => (
              <TextField
                label="Nom"
                required={true}
                disabled={loading}
                {...field}
                error={error !== undefined}
                helperText={
                  error?.type === "required" ? "Champ requis" : error?.message
                }
              />
            )}
          />
        ) : null}
        <Stack direction={"row"} spacing={2}>
          <Controller
            name="data_url"
            control={control}
            rules={{ required: true }}
            render={({ field, fieldState: { error } }) => (
              <TextField
                label="Data URL"
                required={true}
                disabled={loading}
                {...field}
                error={error !== undefined}
                helperText={
                  error?.type === "required" ? "Champ requis" : error?.message
                }
                fullWidth
              />
            )}
          />
          <Box minWidth={200}>
            <Controller
              name="refresh_period"
              control={control}
              rules={{ required: true }}
              render={({ field, fieldState: { error } }) => (
                <FormControl fullWidth>
                  <InputLabel id="update-freq-label">
                    Fréquence de mise à jour
                  </InputLabel>
                  <Select
                    labelId="update-freq-label"
                    label="Fréquence de mise à jour"
                    {...field}
                    error={error !== undefined}
                    disabled={loading}
                  >
                    <MenuItem value={"daily"}>Journalier</MenuItem>
                    <MenuItem value={"weekly"}>Hebdomadaire</MenuItem>
                    <MenuItem value={"monthly"}>Mensuel</MenuItem>
                  </Select>
                  {error ? (
                    <FormHelperText error={true}>
                      {error?.message}
                    </FormHelperText>
                  ) : null}
                </FormControl>
              )}
            />
          </Box>
        </Stack>
        <Controller
          name="description"
          control={control}
          render={({ field, fieldState: { error } }) => (
            <TextField
              label="Description"
              disabled={loading}
              {...field}
              error={error !== undefined}
              helperText={error?.message}
              fullWidth
            />
          )}
        />
        <Box display={"flex"} flexDirection={"row-reverse"}>
          <LoadingButton
            variant="contained"
            startIcon={<SaveIcon />}
            loading={loading}
            type="submit"
            disabled={!isDirty}
          >
            {dataService ? "Sauvegarder" : "Ajouter"}
          </LoadingButton>
        </Box>
      </Stack>
    </form>
  );
}
