import {
  TableContainer,
  Paper,
  Table,
  TableHead,
  TableRow,
  TableCell,
  TableBody,
  CircularProgress,
  Typography,
  IconButton,
} from "@mui/material";
import CheckIcon from "@mui/icons-material/Check";
import CloseIcon from "@mui/icons-material/Close";
import ScheduleIcon from "@mui/icons-material/Schedule";
import Visibility from "@mui/icons-material/Visibility";
import VisibilityOff from "@mui/icons-material/VisibilityOff";
import { ImportProcess } from "@/types";
import { useGetImportProcesses } from "@/hooks/useGetImportProcesses";
import { ErrorScreen } from "./ErrorScreen";
import { LoadingScreen } from "./LoadingScreen";
import { downloadFile, openFile } from "@/utils";
import { ButtonTooltip } from "./ButtonTooltip";

export interface ImprotProcessTableProps {
  dataServiceEid?: number;
  recipeEid?: number;
  projectEid?: number;
}

async function showFile(url?: string) {
  if (url) {
    const fileToOpen = await downloadFile(url);
    openFile(fileToOpen);
  }
}

export function ImportProcessTable(props: ImprotProcessTableProps) {
  const {
    data: importProcessList,
    error,
    loading,
  } = useGetImportProcesses(props);

  if (error) {
    return <ErrorScreen message={error} />;
  }

  if (loading) {
    return <LoadingScreen />;
  }

  if (!importProcessList) {
    return <ErrorScreen />;
  }

  if (importProcessList.length === 0) {
    return (
      <Typography variant="subtitle1" color={"GrayText"}>
        Aucun import process
      </Typography>
    );
  }
  return (
    <TableContainer component={Paper}>
      <Table sx={{ minWidth: 650 }} aria-label="simple table">
        <TableHead>
          <TableRow>
            <TableCell>eid</TableCell>
            <TableCell>Date</TableCell>
            <TableCell>Recette</TableCell>
            <TableCell>État</TableCell>
            <TableCell>Projet</TableCell>
            <TableCell width={50}>Dataset</TableCell>
            <TableCell width={50}>Log</TableCell>
          </TableRow>
        </TableHead>
        <TableBody>
          {importProcessList.map((row, i) => (
            <ImportProcessTableRow key={i} importProcess={row} />
          ))}
        </TableBody>
      </Table>
    </TableContainer>
  );
}

function ImportProcessTableRow({
  importProcess,
}: {
  importProcess: ImportProcess;
}) {
  const hasDataset = importProcess.dataset_url != null;
  const hasLog = importProcess.log_url != null;

  return (
    <TableRow key={importProcess.eid}>
      <TableCell component="th" scope="row">
        {importProcess.eid}
      </TableCell>
      <TableCell>{importProcess.date}</TableCell>
      <TableCell>{importProcess.recipe}</TableCell>
      <TableCell>
        <StateIcon state={importProcess.state} />
      </TableCell>
      <TableCell>{importProcess.project}</TableCell>
      <TableCell>
        <ButtonTooltip
          title={
            hasDataset
              ? "Voir le fichier de Dataset"
              : "Aucun Dataset disponible"
          }
        >
          <IconButton
            color="primary"
            disabled={!hasDataset}
            onClick={() => showFile(importProcess.dataset_url)}
          >
            {hasDataset ? <Visibility /> : <VisibilityOff />}
          </IconButton>
        </ButtonTooltip>
      </TableCell>
      <TableCell>
        <ButtonTooltip
          title={hasLog ? "Voir le fichier de log" : "Aucun log disponible"}
        >
          <IconButton
            color="primary"
            disabled={!hasLog}
            onClick={() => showFile(importProcess.log_url)}
          >
            {hasLog ? <Visibility /> : <VisibilityOff />}
          </IconButton>
        </ButtonTooltip>
      </TableCell>
    </TableRow>
  );
}

function StateIcon({ state }: { state: ImportProcess["state"] }) {
  if (state === "successful") {
    return <CheckIcon />;
  }
  if (state === "error") {
    return <CloseIcon color="error" />;
  }
  if (state === "waiting") {
    return <ScheduleIcon color="warning" />;
  }
  if (state === "ongoing") {
    return <CircularProgress size={20} />;
  }
}
