import { Stack, TextField, Box, Switch, FormControlLabel } from "@mui/material";
import SaveIcon from "@mui/icons-material/Save";
import { Project } from "@/types";
import { Controller, SubmitHandler, useForm } from "react-hook-form";
import { useState } from "react";
import { useRouter } from "next/navigation";
import { LoadingButton } from "@mui/lab";
import { useHandleAuthErrors } from "@/hooks/useHandleAuthErrors";
import { useApiCreateProject, useApiUpdateProject } from "@/api/cubicweb";
import { FileField } from "./FileField";

export interface ProjectFormProps {
  project?: Project;
}

export function ProjectForm({ project }: ProjectFormProps) {
  const [loading, setLoading] = useState(false);
  const {
    handleSubmit,
    control,
    formState: { isDirty },
    reset,
  } = useForm<Project>({
    defaultValues: project ?? { activated: false },
  });
  const handleAuthErrors = useHandleAuthErrors();
  const createProject = useApiCreateProject();
  const updateProject = useApiUpdateProject();

  const router = useRouter();

  const onSubmit: SubmitHandler<Project> = async (data) => {
    setLoading(true);
    try {
      if (data.eid === undefined) {
        const eid = await createProject(data);
        setLoading(false);
        router.push(`/project/${eid}`);
      } else {
        await updateProject(data);
        reset(data);
        setLoading(false);
      }
    } catch (e) {
      handleAuthErrors(e);
    }
  };

  return (
    <form onSubmit={handleSubmit(onSubmit)}>
      <Stack spacing={2}>
        {!project ? (
          <Controller
            name="name"
            control={control}
            rules={{ required: true }}
            render={({ field, fieldState: { error } }) => (
              <TextField
                label="Nom"
                required={true}
                disabled={loading}
                {...field}
                error={error !== undefined}
                helperText={
                  error?.type === "required" ? "Champ requis" : error?.message
                }
              />
            )}
          />
        ) : null}
        <Stack direction={"row"}>
          <Controller
            name="activated"
            control={control}
            render={({ field }) => (
              <Box width={150} display={"flex"} alignItems={"center"}>
                <FormControlLabel
                  control={
                    <Switch
                      {...field}
                      checked={field.value}
                      disabled={loading}
                    />
                  }
                  label={field.value ? "Actif" : "Inactif"}
                />
              </Box>
            )}
          />
          <Controller
            name="virtuoso_url"
            control={control}
            rules={{ required: true }}
            render={({ field, fieldState: { error } }) => (
              <TextField
                label="Virtuoso URL"
                required={true}
                disabled={loading}
                {...field}
                error={error !== undefined}
                helperText={
                  error?.type === "required" ? "Champ requis" : error?.message
                }
                fullWidth
              />
            )}
          />
        </Stack>
        <Controller
          name="ontology_file"
          control={control}
          render={({ field }) => (
            <FileField label="Ontologie" {...field} disabled={loading} />
          )}
        />
        <Controller
          name="shacl_files"
          control={control}
          render={({ field }) => (
            <FileField label="SHACL" {...field} disabled={loading} />
          )}
        />
        <Box display={"flex"} flexDirection={"row-reverse"}>
          <LoadingButton
            variant="contained"
            startIcon={<SaveIcon />}
            type="submit"
            loading={loading}
            disabled={!isDirty}
          >
            {project ? "Sauvegarder" : "Ajouter"}
          </LoadingButton>
        </Box>
      </Stack>
    </form>
  );
}
